@@ -1,10 +1,7 @@
-#[macro_use]
-extern crate derive_more;
-
 use chrono::NaiveDateTime;
 use sqlx::{error::Error as DBError, Pool, Sqlite, SqlitePool};
 use std::num::NonZeroU64;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::sync::{Arc, RwLock};
 
 mod db;
@@ -13,6 +10,7 @@ pub mod migration;
 pub use event::*;
 
 const ZERO: i64 = 0;
+const EMPTY_STR: &str = "";
 
 #[derive(Debug, Clone)]
 pub struct Historian {
@@ -24,8 +22,8 @@ async fn raw_record_event(pool: &Pool<Sqlite>, event: DBEvent<'_>) -> Result<i64
     sqlx::query!(
         "
         INSERT INTO events
-        (script_id, type, cmd, args, content, time, main_event_id)
-        VALUES(?, ?, ?, ?, ?, ?, ?)
+        (script_id, type, cmd, args, content, time, main_event_id, dir)
+        VALUES(?, ?, ?, ?, ?, ?, ?, ?)
         ",
         event.script_id,
         event.ty,
@@ -33,7 +31,8 @@ async fn raw_record_event(pool: &Pool<Sqlite>, event: DBEvent<'_>) -> Result<i64
         event.args,
         event.content,
         event.time,
-        event.main_event_id
+        event.main_event_id,
+        event.dir
     )
     .execute(pool)
     .await?;
@@ -50,6 +49,7 @@ struct DBEvent<'a> {
     cmd: &'a str,
     time: NaiveDateTime,
     args: Option<&'a str>,
+    dir: Option<&'a str>,
     content: Option<&'a str>,
     main_event_id: i64,
 }
@@ -63,12 +63,17 @@ impl<'a> DBEvent<'a> {
             main_event_id: ZERO,
             content: None,
             args: None,
+            dir: None,
         }
     }
     fn args(mut self, value: &'a str) -> Self {
         self.args = Some(value);
         self
     }
+    fn dir(mut self, value: &'a str) -> Self {
+        self.dir = Some(value);
+        self
+    }
     fn content(mut self, value: &'a str) -> Self {
         self.content = Some(value);
         self
@@ -81,13 +86,19 @@ impl<'a> DBEvent<'a> {
 
 macro_rules! select_last_arg {
     ($select:literal, $script_id:expr, $offset:expr, $limit:expr) => {{
+        select_last_arg!($select, $script_id, $offset, $limit, "", )
+    }};
+    ($select:literal, $script_id:expr, $offset:expr, $limit:expr, $where:literal, $($var:expr),*) => {{
         static EXEC_TY: &'static str = EventType::Exec.get_str();
         sqlx::query!(
             "
             WITH args AS (
                 SELECT args, max(time) as time FROM events
-                WHERE type = ? AND script_id = ? AND NOT ignored
-                GROUP BY args
+                WHERE type = ? AND script_id = ? AND NOT ignored "
+                +
+                $where
+                +
+                " GROUP BY args
                 ORDER BY time DESC LIMIT ? OFFSET ?
             ) SELECT "
                 + $select
@@ -95,8 +106,9 @@ macro_rules! select_last_arg {
             ",
             EXEC_TY,
             $script_id,
+            $($var, )*
             $limit,
-            $offset
+            $offset,
         )
     }};
 }
@@ -177,7 +189,7 @@ impl Historian {
         let mut db_event = DBEvent::new(event.script_id, time, &ty, &cmd);
         let id = match &event.data {
             EventData::Write | EventData::Read => self.raw_record(db_event).await?,
-            EventData::Exec { content, args } => {
+            EventData::Exec { content, args, dir } => {
                 let mut content = Some(*content);
                 let last_event = sqlx::query!(
                     "
@@ -197,7 +209,9 @@ impl Historian {
                     }
                 }
                 db_event.content = content;
-                self.raw_record(db_event.args(args)).await?
+                let dir = dir.map(|p| p.to_string_lossy()).unwrap_or_default();
+                self.raw_record(db_event.dir(dir.as_ref()).args(args))
+                    .await?
             }
             EventData::ExecDone {
                 code,
@@ -222,31 +236,45 @@ impl Historian {
         Ok(id)
     }
 
-    pub async fn last_args(&self, id: i64) -> Result<Option<String>, DBError> {
+    pub async fn previous_args(
+        &self,
+        id: i64,
+        dir: Option<&Path>,
+    ) -> Result<Option<String>, DBError> {
         let ty = EventType::Exec.get_str();
+        let no_dir = dir.is_none();
+        let dir = dir.map(|p| p.to_string_lossy());
+        let dir = dir.as_ref().map(|p| p.as_ref()).unwrap_or(EMPTY_STR);
         let res = sqlx::query!(
             "
             SELECT args FROM events
             WHERE type = ? AND script_id = ? AND NOT ignored
+            AND (? OR dir = ?)
             ORDER BY time DESC LIMIT 1
             ",
             ty,
-            id
+            id,
+            no_dir,
+            dir
         )
         .fetch_optional(&*self.pool.read().unwrap())
         .await?;
         Ok(res.map(|res| res.args.unwrap_or_default()))
     }
 
-    pub async fn last_args_list(
+    pub async fn previous_args_list(
         &self,
         id: i64,
         limit: u32,
         offset: u32,
+        dir: Option<&Path>,
     ) -> Result<impl ExactSizeIterator<Item = String>, DBError> {
         let limit = limit as i64;
         let offset = offset as i64;
-        let res = select_last_arg!("args", id, offset, limit)
+        let no_dir = dir.is_none();
+        let dir = dir.map(|p| p.to_string_lossy());
+        let dir = dir.as_ref().map(|p| p.as_ref()).unwrap_or(EMPTY_STR);
+        let res = select_last_arg!("args", id, offset, limit, "AND (? OR dir = ?)", no_dir, dir)
             .fetch_all(&*self.pool.read().unwrap())
             .await?;
         Ok(res.into_iter().map(|res| res.args.unwrap_or_default()))
@@ -398,12 +426,13 @@ impl Historian {
                     SELECT id FROM events
                     WHERE script_id = ?
                       AND args = e.args
+                      AND dir = e.dir
                     ORDER BY time DESC
                     LIMIT 1
                   )
                 FROM
                   (
-                    SELECT distinct args
+                    SELECT distinct args, dir
                     FROM events
                     WHERE script_id = ?
                       AND NOT ignored
